; Castlevania 1 NES to PCE rom hack.
; Version 0.44
; 
; Assemble with PCEAS 3.21 that comes with HuC (www.zeograd.com)
;


;############################################## 
;
;
;
  
    ;internal variables 
 
  .include "VAR.asm"

  .code
  .bank $00
  .org $e000
  
_RESET: 
    sei                       ;you better know this one ;)
    lda #$f8                  
    tam #$01                  ;map ram
    tam #$00
    lda #$ff                              
    tam #$03                  ;map hardware bank
    csh                       ;set 7.16mhz mode
    cld                       ;clear decimal flag 
    ldx #$ff                  ;initialize stack
    txs

    lda $6000                 ;clear pending VDC interrupts
    st0 #$05                  ;enabled vblank/hysnc flag; disabled BG/spr
    st1 #$0c
    st2 #$00
    st0 #$06                  ;setup hsync for first active scanline
    st1 #$40
    st2 #$00
    stz $7403                 ;clear pending timer interrupts
    lda #$F8                  ;enable interrupts via mask
    sta $7402
    lda #$39
    sta $6c00                 ;set timer to 59392 cycles
    lda #$01
    sta $6c01                 ;enable TIMER

    clx
.loop                         ;initialize audio hardware
    stx $6800
    stz $6801
    stz $6802
    stz $6803
    stz $6804
    stz $6805
    stz $6806
    stz $6807
    stz $6808
    stz $6809
    inx
    cpx #$06
    bcc .loop

    
 
    lda #$04                  ;initialize the VCE
    sta $6400
    
    stz $0000
    tii $0000, $0001, $2000   ;clear ram
    st0 #$00
    st1 #$00
    st2 #$00
    st0 #$02
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    st0 #$00
    st1 #$00
    st2 #$00
    st0 #$02
    jsr init_VDC              ;init display parameters
    lda #$01        
    tam #$02                  ;setup our back end code bank to $4000

    lda #$ff
    sta $6801
    stz $6800    
    lda #$40
    sta $6804
    stz $6804
    tin duty_cycle0,$6806,$20 ;update waveform buffer
    lda #$cc
    sta $6805
    lda #$01
    sta $6800
    lda #$40
    sta $6804
    stz $6804
    tin duty_cycle0,$6806,$20 ;update waveform buffer
    lda #$cc
    sta $6805
    lda #$02
    sta $6800
    lda #$40
    sta $6804
    stz $6804
    tin triangle_cycle,$6806,$20  ;update waveform buffer
    lda #$ff
    sta $6805
    lda #$05
    sta $6800
    stz $804
    lda #$99
    sta $6805
    

    stz _spr_lsb
    lda #$7F
    sta _spr_msb
    st0 #$0f
    st1 #$10
    st2 #$00
  
    jmp __prep_rom            ;Initialization done.
  
init_VDC:
  .include "INIT_VDC.asm"
    
    ;// Interrupt startup handlers

_IRQ_2:
    rti
    
_TIMER:
    pha
    lda #$ff
    tam #$00
    stz $1403
    lda #$f8
    tam #$00
    pla
    rti

_VDC_INT:
    pha
    lda #$ff
    tam #$00
    lda $0000
    lda #$f8
    tam #$00
    pla
    rti
  
_NMI:
    rti
    
  .org $fff6
  
    ;// vector_table

    .dw _IRQ_2
    .dw _VDC_INT
    .dw _TIMER
    .dw _NMI
    .dw _RESET




;############################################## 

  .bank $01
  .org $4000
    
__prep_rom:

    lda #$00                  ;setup startup lower bank
    jsr MPR_REG
    lda #FIXED_BANK           ;setup startup fixed upper bank
    tam #$06
    inc a
    tam #$07
    jmp [$fffe]               ;jump to NES reset vector 
    
    .org $4040

  .include "PPU.asm"
  .include "MAPPER.asm"
  .include "IO.asm"
  .include "APU.asm"
  .include "EQUATE.asm"




;############################################## 
;
; Load the rom into the project file. The file
; allocates banks $02 to $11 (128k).
;
  .bank $02
    
castlevania_rom:  
  .incbin "castlevania(PRG 0).bin"
  
                
  

;############################################## 
;
; This section is for the rom hardware interface modifications.
; This will overwrite sections of "megaman.bin" and allow us
; to keep track of the changes in a easy to locate format.
;


  .bank $11   ;################################

  .org $FFF8                  ;setup internal VDC int
    .dw _INT_VDC
  .org $FFF6                  ;remap BRK to PCE vector
    .dw $C0bb
  .org $FFFA                  ;setup internal TMR int
    .dw _INT_TMR
  .org $FFFC                  ;remap NMI to PCE NMI which not used. 
    _NES_NMI: .dw $c052       ;We'll use this as an indirect jump
  .org $FFFE
    .dw $c008                 ;remap reset to PCE vector
    
    
  .org $fd64
    jsr PPU_2006              ;sta $2006
    
  .org $fd69
    jsr PPU_2006              ;sta $2006
    
  .org $fd7e
    jsr PPU_2007              ;sta $2007
    
  .org $f844
    jsr PPU_REG2              ;lda $2002

  .org $f849
    jsr PPU_2005              ;sta $2005
    
  .org $f84e
    jsr PPU_2005              ;sta $2005  
    
  .org $f855
    jsr PPU_REG0              ;sta $2000
    
  .org $f8a8
    jsr PPU_REG2              ;lda $2002
  
  .org $f8b4
    jsr PPU_REG2              ;lda $2002
    
  .org $f8bb                  
    jsr PPU_REG2              ;lda $2002
    
  .org $f8c7
    jsr PPU_REG2              ;lda $2002

  .org $f8ce
    jsr PPU_2005              ;sta $2005
    
  .org $f8d3
    jsr PPU_2005              ;sta $2005
    
  .org $f8e1
    jsr PPU_REG0              ;sta $2000
    
  .org $f8af
    ldx #$40                  ;ldx #$00             This is the delay for the static window


  .bank $10   ;################################
  
  .org $c009
    cli                       ;sei
    jsr PPU_REG2              ;lda $2002
  
  .org $c00f
    jsr PPU_REG2              ;lda $2002
  
  .org $c0f1
    jsr PPU_REG0              ;sta $2000
  
  .org $c11f
    jsr PPU_REG1              ;sta $2001
    jsr PPU_REG0              ;sta $2000
    
  .org $c199
    jsr MPR_REG               ;sta $c000,y
    jsr MPR_REG               ;sta $c000,y
    
  .org $c087
    jsr PPU_REG1              ;sta $2001
    
  .org $c058
    jsr PPU_REG2              ;lda $2002
    
  .org $c064
    jsr PPU_REG3              ;sta $2003
    
  .org $c069      
    jsr SPR_DMA_Y             ;sty $4014  
    
  .org $c10a
    jsr PPU_REG2              ;lda $2002
    
  .org $c10f
    jsr PPU_2005              ;sta $2005
    
  .org $c114
    jsr PPU_2005              ;sta $2005
    
  .org $c119
    jsr PPU_REG0              ;sta $2000
    
  .org $c0fd
    jsr PPU_REG2              ;lda $2002
    
  .org $c102
    jsr PPU_2006              ;sta $2006
    
  .org $c107
    jsr PPU_2006              ;sta $2006
    
  .org $cb8c
    jsr PPU_2007              ;sta $2007
    
  .org $cb9a
    jsr PPU_2007              ;sta $2007
    
  .org $cb58
    jsr PPU_REG2              ;lda $2002
    
  .org $cb5b
    jsr PPU_2006_x            ;stx $2006
    jsr PPU_2006_y            ;sty $2006
    
  .org $c87e
    jsr IO_PORT
    lda __gamepad
    sta <$04
    stz <$05
    stz <$07
    cly
    clx
    jmp $c8a6 
    
  .org $c0df
    jsr PPU_REG0              ;sta $2000 



  .bank $02   ;################################
  
  .org $8038
    jsr PPU_REG0              ;sta $2000
    
  .org $8018
    jsr PPU_REG0              ;sta $2000
    
  .org $801c
    jsr PPU_REG2              ;lda $2002
    
  .org $8022
    jsr PPU_2006              ;sta $2006
  
  .org $8029
    jsr PPU_2006              ;sta $2006
    
  .org $803e
    jsr PPU_2007              ;sta $2007
    
  .org $8288
    jsr APU_4000_iy           ;sta $4000,y

  .org $86a4
    jsr APU_4000_ix           ;sta $4000,x
 
  .org $85bb
    jsr APU_4000_ix           ;sta $4000,x

  .org $873f
    jsr APU_4002_ix           ;sta $4002,x

  .org $871f
    jsr APU_4003_ix           ;sta $4003,x

  .org $86c2
    jsr APU_4000_ix           ;sta $4000,x
    
  .org $828e
    jsr APU_4001_iy           ;sta $4001,y
    
  .org $84ea
    jsr APU_4000_ix           ;sta $4000,x
    
  .org $851d
    jsr APU_4001_ix           ;sta $4001,x
    
  .org $854c
    jsr APU_4000_ix           ;sta $4000,x
    
  .org $8343
    jsr APU_4000_ix           ;sta $4000,x
    
  .org $82d3
    jsr APU_4001              ;sta $4001
    
  .org $82db
    jsr APU_4002              ;sta $4002
    
  .org $82e3
    jsr APU_4003              ;sta $4003
    
  .org $82f1
    jsr APU_4000              ;sta $4000
    
  .org $8304
    jsr APU_4005_y            ;sty $4005
    
  .org $830c
    jsr APU_4006              ;sta $4006
    
  .org $8314
    jsr APU_4007              ;sta $4007
    
  .org $8322
    jsr APU_4004              ;sta $4004
    
  .org $8433
    jsr APU_4004              ;sta $4004
    
  .org $842b
    jsr APU_4000_ix           ;sta $4000,x
    
  .org $843e
    jsr APU_4000              ;sta $4000
    
  .org $844f
    jsr APU_4000_ix           ;sta $4000,x
    
  .org $8457
    jsr APU_4008              ;sta $4008
    
  .org $845f
    jsr APU_400C              ;sta $400c
    
  .org $8622
    jsr APU_4001_ix           ;sta $4001,x

    
    
  
;############################################## 
;
; Pad rom to 512k
;
  .bank $3f                 ;pad to last bank
  .bank $3e                 ;re-align back to LUT bank

    ;LUTs go here.














