; Duck Tales 2 NES to PCE rom hack.
; Version 0.31
; 
; Assemble with PCEAS 3.21 that comes with HuC (www.zeograd.com)
;


;############################################## 
;
;
;
  
    ;internal variables 
 
  .include "VAR.asm"

  .code
  .bank $00
  .org $e000
  
_RESET: 
    sei                       ;you better know this one ;)
    lda #$f8                  
    tam #$01                  ;map ram
    tam #$00
    lda #$ff                              
    tam #$03                  ;map hardware bank
    csh                       ;set 7.16mhz mode
    cld                       ;clear decimal flag 
    ldx #$ff                  ;initialize stack
    txs

    lda $6000                 ;clear pending VDC interrupts
    st0 #$05                  ;enabled vblank/hysnc flag; disabled BG/spr
    st1 #$0c
    st2 #$00
    st0 #$06                  ;setup hsync for first active scanline
    st1 #$40
    st2 #$00
    stz $7403                 ;clear pending timer interrupts
    lda #$F8                  ;enable interrupts via mask
    sta $7402
    lda #$39
    sta $6c00                 ;set timer to 59392 cycles
    lda #$01
    sta $6c01                 ;enable TIMER

    clx
.loop                         ;initialize audio hardware
    stx $6800
    stz $6801
    stz $6802
    stz $6803
    stz $6804
    stz $6805
    stz $6806
    stz $6807
    stz $6808
    stz $6809
    inx
    cpx #$06
    bcc .loop

    
 
    lda #$04                  ;initialize the VCE
    sta $6400
    
    stz $0000
    tii $0000, $0001, $2000   ;clear ram
    st0 #$00
    st1 #$00
    st2 #$00
    st0 #$02
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    st0 #$00
    st1 #$00
    st2 #$00
    st0 #$02
    jsr init_VDC              ;init display parameters
    lda #$01        
    tam #$02                  ;setup our back end code bank to $4000

    lda #$ff
    sta $6801
    stz $6800    
    lda #$40
    sta $6804
    stz $6804
    tin duty_cycle0,$6806,$20 ;update waveform buffer
    lda #$cc
    sta $6805
    lda #$01
    sta $6800
    lda #$40
    sta $6804
    stz $6804
    tin duty_cycle0,$6806,$20 ;update waveform buffer
    lda #$cc
    sta $6805
    lda #$02
    sta $6800
    lda #$40
    sta $6804
    stz $6804
    tin triangle_cycle,$6806,$20  ;update waveform buffer
    lda #$ff
    sta $6805
    lda #$05
    sta $6800
    stz $804
    lda #$99
    sta $6805
    

    stz _spr_lsb
    lda #$7F
    sta _spr_msb
    st0 #$0f
    st1 #$10
    st2 #$00
  
    jmp __prep_rom            ;Initialization done.
  
init_VDC:
  .include "INIT_VDC.asm"
    
    ;// Interrupt startup handlers

_IRQ_2:
    rti
    
_TIMER:
    pha
    lda #$ff
    tam #$00
    stz $1403
    lda #$f8
    tam #$00
    pla
    rti

_VDC_INT:
    pha
    lda #$ff
    tam #$00
    lda $0000
    lda #$f8
    tam #$00
    pla
    rti
  
_NMI:
    rti
    
  .org $fff6
  
    ;// vector_table

    .dw _IRQ_2
    .dw _VDC_INT
    .dw _TIMER
    .dw _NMI
    .dw _RESET




;############################################## 

  .bank $01
  .org $4000
    
__prep_rom:

    lda #$00                  ;setup startup lower bank
    jsr MPR_REG
    lda #FIXED_BANK           ;setup startup fixed upper bank
    tam #$06
    inc a
    tam #$07
    jmp [$fffe]               ;jump to NES reset vector 
    
    .org $4040

  .include "PPU.asm"
  .include "MAPPER.asm"
  .include "IO.asm"
  .include "APU.asm"
  .include "EQUATE.asm"




;############################################## 
;
; Load the rom into the project file. The file
; allocates banks $02 to $11 (128k).
;
  .bank $02
    
DuckTales2_rom:  
  .incbin "Duck_tales2.bin"
  
                
  

;############################################## 
;
; This section is for the rom hardware interface modifications.
; This will overwrite sections of "megaman.bin" and allow us
; to keep track of the changes in a easy to locate format.
;


  .bank $11   ;################################

  .org $FFF8                  ;setup internal VDC int
    .dw _INT_VDC
  .org $FFF6                  ;remap BRK to PCE vector
    .dw $C088
  .org $FFFA                  ;setup internal TMR int
    .dw _INT_TMR
  .org $FFFC                  ;remap NMI to PCE NMI which not used. 
    _NES_NMI: .dw $c651       ;We'll use this as an indirect jump
  .org $FFFE
    .dw $c088                 ;remap reset to PCE vector
    
    
    


  .bank $10   ;################################

  .org $c088
    nop                       ;sei

  .org $c08b
    jsr PPU_2000_x
    jsr PPU_2001_x

  .org $c095
    jsr BIT_PPU_2002

  .org $c09a
    jsr BIT_PPU_2002
    
  .org $cf56
    jsr PPU_2002_x
    jsr PPU_2006_y
    jsr PPU_2006

  .org $cf71
    jsr PPU_2007
    
  .org $d398
    jsr MPR_REG
  
  .org $ce73
    jsr PPU_2002_x
    
  .org $ce7a
    jsr PPU_2006_y
    jsr PPU_2006
    
  .org $cf71
    jsr PPU_2007

  .org $ce99
    jsr PPU_2007

  .org $cec3
    jsr PPU_2000
    
  .org $cec7
    jsr PPU_2002

  .org $cecc
    jsr PPU_2006
    
  .org $ced2
    jsr PPU_2006
    
  .org $ced8
    jsr PPU_2007

  .org $c658
    jsr PPU_2000
    
  .org $c661
    jsr PPU_2003

  .org $c666
    jsr SPR_DMA_A
    
  .org $c66d
    jsr PPU_2002_y
    jsr PPU_2006_x
    jsr PPU_2006

  .org $c67b
    jsr PPU_2007

  .org $c687
    jsr PPU_2006_x
    jsr PPU_2006
    jsr PPU_2006
    jsr PPU_2006

  .org $c6a2
    jsr PPU_2000
    
  .org $c6a6
    jsr PPU_2002

  .org $c6ac
    jsr PPU_2006
    
  .org $c6b3
    jsr PPU_2006

  .org $c6ba
    jsr PPU_2007

  .org $c6cb
    jsr PPU_2005
    jsr PPU_2005

  .org $c6d3
    jsr PPU_2000

  .org $c6d8
    jsr PPU_2001

  .org $c70b
    jsr BIT_PPU_2002

  .org $c710
    jsr BIT_PPU_2002
    
  .org $c715
    jsr PPU_2001_x

  .org $c74a
    jsr PPU_2000_y        ; I changed this around. It's supposed to be 2006,2006,2000 in that order.
    jsr PPU_2006_x
    jsr PPU_2006

  .org $c757
    jsr PPU_2005_x
    jsr PPU_2005

  .org $c766
    jsr PPU_2000_y        ; I changed this around. It's supposed to be 2005,2005,2000 in that order.
    jsr PPU_2005_x
    jsr PPU_2005

  .org $c776
    jsr PPU_2001

  .org $c7b7
    jsr PPU_2000

  .org $c7c3
    jsr PPU_2002

  .org $c7cc
    jsr PPU_2000

  .org $d329
    jsr IO_PORT
    lda __gamepad
    sta <$22
    stz <$25
    jmp $d34a

  .org $c0a6
    jsr APU_4000_ix

  .org $c0ae
    jsr APU_4001
    jsr APU_4005
    
  .org $c0b6
    jsr APU_4015

  .org $c0bb
    jsr APU_4017

    





  .bank $02   ;################################

  .org $8233
    jsr APU_4000_iy

  .org $823b
    jsr APU_4000
    jsr APU_4004

  .org $8243
    jsr APU_4008
    
  .org $8248
    jsr APU_400C

  .org $824d
    jsr APU_4015
    
  .org $8159
    jsr APU_4000_iy

  .org $8252
    jsr APU_4017

  .org $834d
    jsr APU_400C

  .org $8168
    jsr APU_4001_iy

  .org $818f
    jsr APU_4002_iy
    
  .org $8198
    jsr APU_4003_iy




    





    
    
;############################################## 
;
; Pad rom to 512k
;
  .bank $3f                 ;pad to last bank
  .bank $3e                 ;re-align back to LUT bank

    ;LUTs go here.














