;################################################################################
; 
; PPU simulator 
;

;################################################################################
;
PPU_REG0:                     ; To do.
PPU_2000:                     ; $2000 (byte) -w
                              ; D7 = 0=disable interrutps / 1=enable interrupts
                              ; D6 = null
                              ; D5 = 0=8x8 sprites / 1=8x16 sprites
                              ; D4 = 0=$0000 BG pattern table / 1=$1000 BG pattern table
                              ; D3 = 0=$0000 SPR pattern table / 1=$1000 SPR pattern table
                              ; D2 = 0=PPU increment 1 / 1=PPU increment 32
                              ; D1-D0 = Name table address:
                              ;         00=$2000
                              ;         01=$2400
                              ;         10=$2800
                              ;         11=$2C00
              
    sta _ppu_reg0
    rts
                 
PPU_2000_y:
		sty _ppu_reg0
		rts
PPU_2000_x:
		stx _ppu_reg0
		rts


PPU_REG1:                     ; To do.
PPU_2001:                     ; $2001 (byte) -w
                              ; D7-D5 = null (BG color)
                              ; D4 = 0=sprites off / 1=sprites on
                              ; D3 = 0=BG off / 1=BG on
                              ; D2 = null (sprite clipping) set to BG clipping
                              ; D1 = 0=BG clipping / 1=no clipping ( 256-8 left side)
                              ; D0 = null ( color / mono)
    phx
    sta _ppu_reg1
    lsr a
    lsr a
    lsr a
    tax
    lda disp_tbl,x
    sta _cr_reg
    plx
    lda _ppu_reg1
    rts
disp_tbl: .db $0c,$8c,$4c,$cc

PPU_2001_x:
		pha
		txa
		jsr PPU_2000
		txa
		pla
		rts

;################################################################################
;
PPU_REG2:                     ; 
PPU_2002:                     ; $2002 (byte) -r
                              ; D7 = 0=active / 1=vblank
                              ; D6 = sprite 0 occurance
                              ;      0= sprite not encountered
                              ;      1= sprite on scanline encountered
                              ; D5 = 0=not sprite overflow / 1=sprite overflow
                              ; D4 = (null) 0=vram writes enabled / 1=vram writes ignored
                              ; *note: testing D4 is not needed for PCE vram writes

    sei                       ; VDC IRQ can happen between reading and resetting status reg
    lda _ppu_reg2
    stz _ppu_reg2
    stz _ppu_vaddr_latch
    cli
    rts

BIT_PPU_2002:
    sei                       ; VDC IRQ can happen between reading and resetting status reg
    bit _ppu_reg2
    stz _ppu_reg2
    stz _ppu_vaddr_latch
    cli
    rts
PPU_2002_x:
    sei                       ; VDC IRQ can happen between reading and resetting status reg
    ldx _ppu_reg2
    stz _ppu_reg2
    stz _ppu_vaddr_latch
    cli
    rts
PPU_2002_y:
    sei                       ; VDC IRQ can happen between reading and resetting status reg
    ldy _ppu_reg2
    stz _ppu_reg2
    stz _ppu_vaddr_latch
    cli
    rts



PPU_REG3:                     ;
PPU_2003:                     ; $2003 (byte) -w
                              ; Sprite table offset
                              ;
    sta _spr_lsb
    pha
    and #$03
    asl a
    sta _spr_pos
    pla                     
    rts                             

                
;################################################################################
;
PPU_REG4:                     ;
PPU_2004:                     ; $2004 (byte) -r/w
                              ; Sprite table data
                              ;
                              ; Note: doesn't look like this is used in the game.
    rts	; for now

                  
;################################################################################
;
SPR_DMA_Y:
	sty spr_dma_addr
	jmp SPR_DMA
	
SPR_DMA_A:
	sta spr_dma_addr
	jmp SPR_DMA
	
SPR_DMA:                      ;
                              ; $4014 (byte) -w
                              ; copy $100 bytes to SPR ram
                              ;
    
    phy
    pha
    phx
    sei
    ldy #$01
    sty _internal_halt
    st0 #$00
    st1 #$00
    st2 #$7f
    st0 #$02
    ldy #$02
    sty _vdc_reg
    cli
    ldy <$00
    phy			
    ldy <$01
    phy			
    ldy <$02
    phy
    ldy <$03
    phy
    ldy <$04
    phy
    ldy <$05
    phy
    ldy <$06
    phy
    ldy <$07
    phy
 
   	lda spr_dma_addr
   
    sta <$01									; Setup MSB for pointers
    sta <$03
    sta <$05
    sta <$07
		
		stz <$00									; Setup LSB for pointers
		lda #$01
		sta <$02
		lda #$02
		sta <$04
		lda #$03
		sta <$06

		lda [$00]
		sta sprite_zero
		
    
    cly
.loop
    lda [$04],y               ; ATTRIB
    tax												
    bit #$80
    bne .y_flip
    clc                       
    lda [$00],y								; Y
    adc #$41
    sta $6002                 ; write Y 
    cla
    adc #$00
    sta $6003
    txa
    bit #$40
    bne .x_flip
    pha												; push ATTRIB
    lda [$06],y               ; X
    clc
    adc #$20
    sta $6002                 ; write X
    cla
    adc #$00
    sta $6003
    lda [$02],y								; ADDR
    tax
    lda spr_addr_low,x						
    sta $6002									; write ADDR
    lda spr_addr_hi,x							
		sta $6003
    plx               				; pull ATTRIB	
    lda spr_attrb_lut,x				
    sta $6002									; write ATTRIB
    st2 #$00									; No flip
    jmp .lp_cnt

.x_flip
		pha												; push ATTRIB
    lda [$06],y               ; X
    clc
    adc #$18
    sta $6002                 ; 
    cla
    adc #$00
    sta $6003
    lda [$02],y								; ADDR
    tax
    lda spr_addr_low,x						
    sta $6002									; write ADDR
    lda spr_addr_hi,x							
		sta $6003
    plx												; pull ATTRIB
    lda spr_attrb_lut,x				 
    sta $6002									; write ATTRIB
    st2 #$08									; X flip
    jmp .lp_cnt

.y_flip                       
    clc
    lda [$00],y								; Y
    adc #$39
    sta $6002                 ; write Y 
    cla
    adc #$00
    sta $6003
    txa
    bit #$40
    bne .x_flip2
    pha												; push ATTRIB
    lda [$06],y               ; X
    clc
    adc #$20
    sta $6002                 ; write X 
    cla
    adc #$00
    sta $6003
    lda [$02],y								; ADDR
    tax
    lda spr_addr_low,x						
    sta $6002									; write ADDR
    lda spr_addr_hi,x							
		sta $6003
    plx                				; pull ATTRIB
    lda spr_attrb_lut,x				
    sta $6002									; write ATTRIB 
    st2 #$80									; Y flip
    jmp .lp_cnt
    
.x_flip2
		pha												; push ATTRIB      
    lda [$06],y               ; X
    clc
    adc #$18
    sta $6002                 ; write X 
    cla
    adc #$00
    sta $6003
    lda [$02],y								; ADDR
    tax
    lda spr_addr_low,x						
    sta $6002									; write ADDR
    lda spr_addr_hi,x							
		sta $6003
		plx												; pull ATTRIB
    lda spr_attrb_lut,x
    sta $6002									; write ATTRIB
    st2 #$88									; X & Y flip
  
.lp_cnt 
    iny
    iny
    iny
    iny
    beq .done
    jmp .loop

.done 
    ply
    sty <$07
    ply
    sty <$06
    ply
    sty <$05
    ply
    sty <$04
    ply
    sty <$03
    ply
    sty <$02
    ply
    sty <$01    
    ply
    sty <$00
    sei
    stz _internal_halt
    cli
    plx
    pla
    ply
    rts

                                              

;################################################################################
;
PPU_2005:                     ; To do.
                              ; PPU scroll reg (word)
                              ; 
                              ; 1st byte = horizontal scroll position
                              ; 2nd byte = vertical scroll position
                              ;   
    ;pha
    ;lda _ppu_vaddr_latch
		tst #$ff, _ppu_vaddr_latch
    beq .hscroll
    
.vscroll
    stz _ppu_vaddr_latch      ;reset latch to hscroll reg
		sta _y_scroll
		stz _y_scroll+1						

		pha
		cmp #$10
		bcc .no_mirror
		lda #$ef
		sec
		sbc _y_scroll
		cmp #$08
		bcc .no_mirror
		clc
		adc #$41
		sta hsync_mirror
		cla
		adc #$00
		sta hsync_mirror+1
		lda #$01
		sta _hsync_mirror_active
		lda _x_scroll_tmp
		sta _x_scroll
		pla
		rts

.no_mirror
		stz _hsync_mirror_active
		lda _x_scroll_tmp
		sta _x_scroll
		pla
		rts


.hscroll
    inc _ppu_vaddr_latch      ;set latch to vscroll reg
		tst #$01,_ppu_reg0
    bne .uppr
    sta _x_scroll_tmp
    stz _x_scroll_tmp+1
    bra .chk_act_1
.uppr
    sta _x_scroll_tmp
    stz _x_scroll_tmp+1
		inc _x_scroll_tmp+1
.chk_act_1
		pha
		lda _active_xy
		beq .done_X
		lda _x_scroll_tmp
		st0 #$07
		sta $6002
		lda _x_scroll_tmp+1
		sta $6003
		
		
.done_X
		pla
    rts


PPU_2005_x:
		pha
		txa
		jsr PPU_2005
		tax
		pla
		rts


;################################################################################
;
PPU_2006:                     ; To do.
                              ; PPU vram address reg (word)
                              ; MSB/LSB
    
    tst #$ff, _ppu_vaddr_latch
    bne _2006_write
    ;sta _ppu_vaddr_hi
    sta _2006_bffr
    inc _ppu_vaddr_latch
    rts
    
_2006_write:                  ;This really needs to be moved into ram for a self modifying code version.
    stz _ppu_vaddr_latch
    pha
_2006_recalc:									; a "jump to" for recalculating (internal)
    sta _ppu_vaddr_low
    sta _ppu_vaddr_r_low
    stz _ppu_read_delay
    sta _intl_ptrn_vaddr_low
    lda _2006_bffr
    sta _ppu_vaddr_hi
    sta _ppu_vaddr_r_hi
    sta _intl_ptrn_vaddr_hi
    cmp #$10
    bcs .next									;sprites are in bank $0000, tiles in bank $1000
    asl _intl_ptrn_vaddr_low
    rol _intl_ptrn_vaddr_hi		
    asl _intl_ptrn_vaddr_low	
    rol _intl_ptrn_vaddr_hi	
    lda _intl_ptrn_vaddr_hi					
    clc                 			
    adc #$10                  
    sta _intl_ptrn_vaddr_hi
    stz _ppu_index
    pla
    rts
    
.next
    cmp #$20
    bcs .next1
    adc #$40
    sta _intl_ptrn_vaddr_hi
    lda #$02
    sta _ppu_index
    bra .prep
.next1
    cmp #$30
    bcc .map
    cmp #$3f
    bcs .pal
    lda #$0a                  ;else unused ram
    sta _ppu_index
    bra .out
.pal
    lda #$08
    sta _ppu_index
    bra .out
    
    
    
.map
    cmp #$23
    beq .atbl_check
    cmp #$27
    beq .atbl_check
    cmp #$2b
    beq .atbl_check
    cmp #$2f
    beq .atbl_check
.ntbl
		;and #$0f
    sta _ppu_vaddr_hi
    lda #$04
    sta _ppu_index
    bra .out
    
    

.prep
    ;asl _intl_ptrn_vaddr_low
    ;rol _intl_ptrn_vaddr_hi
    
.out
    pla
    rts
  
.atbl_check
    lda _ppu_vaddr_low
    cmp #$c0
    bcs .atbl
    lda _ppu_vaddr_hi
    bra .ntbl 
.atbl
    lda _ppu_vaddr_hi
    sta _ppu_vaddr_hi
    lda #$06
    sta _ppu_index
    bra .out    


;################################################################################
;
PPU_2006_x:
    pha
    txa
    jsr PPU_2006
    pla
    rts 

PPU_2006_y:
    pha
    tya
    jsr PPU_2006
    pla
    rts 


    
;################################################################################
;
PPU_2007:                     ; To do.
                              ; PPU write port (byte)
                              ;
                              ; This is going to need some adaptive code if +32 increment is used.
                              ;
                              ; Current status: +1 increment of vram writes.
                              ; Byte only data and byte offsets present some problems

                    
    phx
    pha
    ldx _ppu_index
    jmp [vram_tbl,x]
     
__unused:    
    pla
    plx
    rts
__pal:
    lda _ppu_vaddr_hi
    cmp #$3f
    bne __unused
    lda _ppu_vaddr_low
    and #$1f
    cmp #$10
    bcc .bg
.else_spr
    tax
    pla
    sta pal_bffr,x
    pha
    lda _pal_lut,x
    sta $6402
    lda #$01
    sta $6403
    pla
    pha
    asl a
    tax
    lda pal_table,x
    sta $6404
    lda pal_table+1,x
    sta $6405
    inc _ppu_vaddr_low
    bne .skip
    inc _ppu_vaddr_hi
.skip
    pla
    plx
    rts
    
    
.bg   
    tax
    pla
    sta pal_bffr,x
    pha
    lda _pal_lut,x
    sta $6402
    stz $6403
    pla
    pha
    asl a
    tax
    lda pal_table,x
    sta $6404
    lda pal_table+1,x
    sta $6405
    inc _ppu_vaddr_low
    bne .skip
    inc _ppu_vaddr_hi
.skip1
    pla
    plx
    rts
_pal_lut: .db $00, $02, $08, $0A, $10, $12, $18, $1A, $20, $22, $28, $2A, $30, $32, $38, $3A
          .db $00, $01, $04, $05, $10, $11, $14, $15, $20, $21, $24, $25, $30, $31, $34, $35
    
    
__att_tbl:     								; Yay!!!

			pha
			phy
			tax
			and #$03
			tay
			lda atr_pal_lut,y
			ora #$05
			sta _pal1
			txa
			and #$0c
			tay
			lda atr_pal_lut,y
			ora #$05
			sta _pal2
			txa
			and #$30
			tay
			lda atr_pal_lut,y
			ora #$05
			sta _pal3
			txa
			and #$c0
			tay
			lda atr_pal_lut,y
			ora #$05
			sta _pal4
			ply


			lda _ppu_vaddr_hi
			cmp #$23								;2000
			beq .atr_tbl0							
			cmp #$27								;2400
			beq .atr_tbl1							
			cmp #$2b								;2800
			beq .atr_tbl0							
			cmp #$2f								;2c00
			beq .atr_tbl1
.error
			pla
			pla
			plx			
			jmp recalc_vaddr				; not an attribute table address							

.atr_tbl1
			lda _ppu_vaddr_low
			cmp #$c0
			bcc .error
			and #$3f
			tax
			lda atr_addr_lsb,x
			clc
			adc #$20
			sei
			sta atr_vaddr
			st0 #$00
			sta $6002
			st0 #$01
			sta $6002
			lda atr_addr_msb,x
			adc #$00
			sta atr_vaddr+1
			sta $6003
			st0 #$00
			sta $6003
			st0 #$02
			lda #$02
			sta _vdc_reg
			cli
			pla
			sta atr_tbl1_bffr,x
			bra .start			


.atr_tbl0
			lda _ppu_vaddr_low
			cmp #$c0
			bcc .error
			and #$3f
			tax
			lda atr_addr_lsb,x
			sei
			sta atr_vaddr
			st0 #$00
			sta $6002
			st0 #$01
			sta $6002
			lda atr_addr_msb,x
			sta atr_vaddr+1
			sta $6003
			st0 #$00
			sta $6003
			st0 #$02		
			lda #$02
			sta _vdc_reg
			cli	
			pla
			sta atr_tbl0_bffr,x

.start
	
			ldx #$02
.loop			
			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal1					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal1					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal2					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal2					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			dex
			beq .next
			sei
			lda atr_vaddr			;5
			st0 #$00					;5
			clc								;2
			adc #$40					;2
			sta $6002					;6
			st0 #$01					;5 *
			sta $6002					;6 *
			lda atr_vaddr+1				;5
			adc #$00					;2
			sta $6003					;6 *
			st0 #$00					;5 * = 22 cycles overhead / 4 = 5.5 cycles per write
			sta $6003					;6
			st0 #$02					;5
			cli
			bra .loop

.next
			sei
			lda atr_vaddr			;5
			st0 #$00					;5
			clc								;2
			adc #$80					;2
			sta $6002					;6
			st0 #$01					;5 *
			sta $6002					;6 *
			lda atr_vaddr+1				;5
			adc #$00					;2
			sta $6003					;6 *
			st0 #$00					;5 * = 22 cycles overhead / 4 = 5.5 cycles per write
			sta $6003					;6
			st0 #$02					;5
			cli

			ldx #$02
.loop1			
			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal3					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal3					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal4					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			lda $6002					;6    1 cycle overhead = 6.5 cycle overhead
			sta $6002					;6 
			lda $6003					;6
			lda _pal4					;5
			sta $6003					;6 = 29 cycles (7.25 NES)

			dex
			beq .end
			
			sei
			lda atr_vaddr					;5
			st0 #$00					;5
			clc								;2
			adc #$c0					;2
			sta $6002					;6
			st0 #$01					;5 *
			sta $6002					;6 *
			lda atr_vaddr+1				;5
			adc #$00					;2
			sta $6003					;6 *
			st0 #$00					;5 * = 22 cycles overhead / 4 = 5.5 cycles per write
			sta $6003					;6
			st0 #$02					;5
			cli
			bra .loop1

.end
		lda _ppu_reg0							;5
		bit #$04									;2
		bne .32inc								;4  = 96 cycles (24cycles)
		inc _ppu_vaddr_low				
		bne .skip
		inc _ppu_vaddr_hi
.skip		
		pla
		plx
		rts
.32inc
		lda _ppu_vaddr_low
		clc
		adc #$20
		sta _ppu_vaddr_low
		bcc .msb_skip
		inc _ppu_vaddr_hi
.msb_skip			
		pla
    plx
    rts






__out01:    
    pla
    plx
    jmp recalc_vaddr
  
__name_tbl:    
		lda _ppu_vaddr_hi					;5
		asl a											;2
		tax												;2
		jmp [name_tbl_check,x]		;7   16
	
_name_tbl_attr_check:		
		lda _ppu_vaddr_low				;5
		cmp #$c0									;2
		bcs __out01								;2/4
		jmp [name_tbl,x]					;7 15

		
__tbl_wr1:
		lda _ppu_vaddr_low
		and #$e0
		sta _temp
		lda _ppu_vaddr_hi
		and #$03
		clc
		adc _temp
		beq .tbl2_1st_row
		lda _temp
		asl a
		sei
		php												; ***
		sta _temp
		lda _ppu_vaddr_low
		and #$1f
		clc
		adc #$20
		adc _temp
		st0 #$00
		sta $6002
		st0 #$01
		sta $6002
		lda _ppu_vaddr_hi		
		and #$03
		plp												; ***
		rol a
		sta $6003
		st0 #$00
		sta $6003
		lda #$02
		sta _vdc_reg		
		st0 #$02
		cli
		pla												; ***
		pha												; ***
		sta $6002
		lda $6003
		and #$30									; There seems to be some garbage being written to this area for some reason - AND $30 just in case
		ora #$05
		sta $6003
		bra .tbl2_inc_addr
		
.tbl2_1st_row
		lda _ppu_vaddr_low
		clc
		sei
		adc #$20
		st0 #$00
		sta $6002
		st0 #$01
		sta $6002
		lda _ppu_vaddr_hi
		and #$03
		sta $6003
		st0 #$00
		sta $6003
		lda #$02
		sta _vdc_reg		
		st0 #$02									;5
		cli
		pla												;4 ***
		pha												;3 ***
		sta $6002									;5
		lda $6003
		and #$30
		ora #$05
		sta $6003
		
.tbl2_inc_addr
		lda _ppu_reg0							;5
		bit #$04									;2
		bne .32inc2								;4  = 96 cycles (24cycles)
		inc _ppu_vaddr_low				
		bne .skip02
		inc _ppu_vaddr_hi
.skip02		
		pla
		plx
		rts
.32inc2
		lda _ppu_vaddr_low
		clc
		adc #$20
		sta _ppu_vaddr_low
		bcc .msb_skip2
		inc _ppu_vaddr_hi
.msb_skip2
		pla
		plx
		rts



__tbl_wr0:
		lda _ppu_vaddr_low
		and #$e0
		sta _temp
		lda _ppu_vaddr_hi
		and #$03
		clc
		adc _temp
		beq .tbl1_1st_row
		lda _temp
		asl a
		sei
		php												; ***
		sta _temp
		lda _ppu_vaddr_low
		and #$1f
		clc
		adc _temp
		st0 #$00
		sta $6002
		st0 #$01
		sta $6002
		lda _ppu_vaddr_hi
		and #$03
		plp												; ***
		rol a
		sta $6003
		st0 #$00
		sta $6003
		lda #$02
		sta _vdc_reg
		st0 #$02
		cli
		pla												; ***
		pha
		sta $6002
		lda $6003
		and #$30
		ora #$05
		sta $6003
		bra .tbl1_inc_addr

.tbl1_1st_row
		sei
		lda _ppu_vaddr_low
		st0 #$00
		sta $6002
		st0 #$01
		sta $6002
		lda _ppu_vaddr_hi
		and #$03
		sta $6003
		st0 #$00
		sta $6003
		lda #$02
		sta _vdc_reg
		st0 #$02
		cli
		pla
		pha
		sta $6002
		lda $6003
		and #$30
		ora #$05
		sta $6003

.tbl1_inc_addr
		lda _ppu_reg0
		bit #$04
		bne .32inc1
		inc _ppu_vaddr_low
		bne .skip01
		inc _ppu_vaddr_hi
.skip01		
		pla
		plx
		rts
.32inc1
		lda _ppu_vaddr_low
		clc
		adc #$20
		sta _ppu_vaddr_low
		bcc .msb_skip1
		inc _ppu_vaddr_hi
.msb_skip1
		pla
		plx
		rts

     
__ptrn_tbl1:
    lda _ppu_vaddr_hi
    cmp #$20
    bcc __2007_tile
    pla
    plx
    jmp recalc_vaddr
 
__2007_tile:
    sei
    lda _intl_ptrn_vaddr_low
    st0 #$00
    sta $6002
    lda _intl_ptrn_vaddr_hi
    sta $6003
    st0 #$00
    st0 #$02
    pla
    pha
    stz $6002
    sta $6003
    cli
    inc _ppu_vaddr_low					; no +32 increment for tiles/sprites.
    bne .skip										; Game doesn't appear to use it for such(as of yet).
    inc _ppu_vaddr_hi
.skip
    inc _intl_ptrn_vaddr_low
    bne .skip1
    inc _intl_ptrn_vaddr_hi
.skip1
    pla
    plx
    rts


__ptrn_tbl0:
    lda _ppu_vaddr_hi
    cmp #$10
    bcc __2007_sprite
    pla
    plx
    jmp recalc_vaddr

__2007_sprite:
    sei
    lda _intl_ptrn_vaddr_low
    st0 #$00
    sta $6002
    lda _intl_ptrn_vaddr_hi
    sta $6003
    st0 #$00
    st0 #$02
    pla
    pha
    stz $6002
    sta $6003
    cli
    inc _ppu_vaddr_low
    bne .skip
    inc _ppu_vaddr_hi
.skip
    inc _intl_ptrn_vaddr_low
    bne .skip1
    inc _intl_ptrn_vaddr_hi
.skip1

.sprite_prep
    tst #$08, _intl_ptrn_vaddr_low
    beq .out
    lda _intl_ptrn_vaddr_low
    clc
    adc #$18
    sta _intl_ptrn_vaddr_low
    bcc .out
    inc _intl_ptrn_vaddr_hi
.out
    pla
    plx
    rts
    
    

PPU_2007_Y:
    pha
    tya
    jsr PPU_2007
    tay
    pla
    rts

vram_tbl:
    .dw __ptrn_tbl0,__ptrn_tbl1,__name_tbl,__att_tbl,__pal,__unused 


;################################################################################
;    
recalc_vaddr:

		pha
		lda _ppu_vaddr_hi
		cmp #$10
		bcs .next0
		jmp .sprites
.next0
		cmp #$20
		bcs .next1
		jmp .tiles
.next1
		cmp #$24
		bcs .next2
		jmp .atr_check1
.next2
		cmp #$28
		bcs .next3
		jmp .atr_check2
.next3
		cmp #$2c
		bcs .next4
		jmp .atr_check3
.next4
		cmp #$30
		beq .pal
		bcs .invalid
		jmp .atr_check4
.pal
    lda #$08
    sta _ppu_index
		pla
		jsr PPU_2007
		rts
		
.invalid
		pla
		rts				



.sprites
		sta _intl_ptrn_vaddr_hi
		lda _ppu_vaddr_low
		asl a
    rol _intl_ptrn_vaddr_hi		
		asl a
    rol _intl_ptrn_vaddr_hi		
		sta _intl_ptrn_vaddr_low
		lda _intl_ptrn_vaddr_hi		
    clc                 			
    adc #$10                  
    sta _intl_ptrn_vaddr_hi
    stz _ppu_index
		pla
		jsr PPU_2007
		rts
		
.tiles
    adc #$40
    sta _intl_ptrn_vaddr_hi
    lda #$02
    sta _ppu_index
		lda _ppu_vaddr_low
		sta _intl_ptrn_vaddr_low
		pla
		jsr PPU_2007
		rts


.atr_check1
		cmp #$23
		bcc .name_table
		lda _ppu_vaddr_low
		cmp #$c0
		bcc .name_table
    lda #$06
    sta _ppu_index
		pla
		jsr PPU_2007
		rts

.atr_check2
		cmp #$27
		bcc .name_table
		lda _ppu_vaddr_low
		cmp #$c0
		bcc .name_table
    lda #$06
    sta _ppu_index
		pla
		jsr PPU_2007
		rts

.atr_check3
		cmp #$2b
		bcc .name_table
		lda _ppu_vaddr_low
		cmp #$c0
		bcc .name_table
    lda #$06
    sta _ppu_index
		pla
		jsr PPU_2007
		rts

.atr_check4
		cmp #$2f
		bcc .name_table
		lda _ppu_vaddr_low
		cmp #$c0
		bcc .name_table
    lda #$06
    sta _ppu_index
		pla
		jsr PPU_2007
		rts
		
		
		
.name_table
		lda #$04
		sta _ppu_index		
		pla
		jsr PPU_2007
		rts
		


;################################################################################
;
PPU_2007_R
															; To do.
		
		lda _ppu_vaddr_r_hi
		cmp #$23
		beq .atr_tbl0
		cmp #$27
		beq .atr_tbl1
		cmp #$2b
		beq .atr_tbl0
		cmp #$2f
		beq .atr_tbl1
		rts

.first_delay									;this isn't the best method, it's just a first time read delay
		lda #$01									;hopefully the game doesn't more than 2 reads at a time
		sta _ppu_read_delay
		cla												;maybe this should be a random value
		
		rts

.atr_tbl0
		lda _ppu_vaddr_r_low
		cmp #$c0
		bcc .out
		tst #$ff, _ppu_read_delay
		beq .first_delay
		and #$3f
		phx
		tax
		lda _ppu_reg0
		bit #$04
		beq .skip
		lda _ppu_vaddr_r_low
		clc
		adc #$20
		sta _ppu_vaddr_r_low
		bcc .no_msb
		inc _ppu_vaddr_r_hi
.no_msb
		lda atr_tbl0_bffr,x
		plx
		rts
.skip
		inc _ppu_vaddr_r_low
		bne .no_msb1
		inc _ppu_vaddr_r_hi
.no_msb1
		lda atr_tbl0_bffr,x
		plx
		rts
.out
		cla
		rts		

.atr_tbl1
		lda _ppu_vaddr_r_low
		cmp #$c0
		bcc .out2
		tst #$ff, _ppu_read_delay
		beq .first_delay
		and #$3f
		phx
		tax
		lda _ppu_reg0
		bit #$04
		beq .skip1
		lda _ppu_vaddr_r_low
		clc
		adc #$20
		sta _ppu_vaddr_r_low
		bcc .no_msb2
		inc _ppu_vaddr_r_hi
.no_msb2
		lda atr_tbl1_bffr,x
		plx
		rts
.skip1
		inc _ppu_vaddr_r_low
		bne .no_msb3
		inc _ppu_vaddr_r_hi
.no_msb3
		lda atr_tbl1_bffr,x
		plx
		rts		
.out2
		cla		
		rts


PPU_4014:
		rts

;################################################################################
;
; internal code
;


pal_table:                    ; A NES to PCE color table 
  .incbin "nes_table.dat"
  
  

_INT_VDC:
		pha
    lda $6000
    and #$20
    beq .active
    jmp .vblank
		
.active
		lda _split_mode
		bne .split_mode
		st0 #$05
		  ;lda _cr_reg
		  ;sta $6002
		st1 #$cc
		stz _2nd_half
    lda _ppu_reg2
    and #$3f                  ;Active display is starting, reset status flag
    sta _ppu_reg2
		st0 #$07
		lda _x_scroll
		sta $6002
		tst #$01, _ppu_reg0
		bne .tbl_offset0
		st2 #$00
		bra .cont0
.tbl_offset0
		st2 #$00	;#$01
.cont0
		st0 #$08
		lda _y_scroll
		clc
		adc #$08
		cmp #$f0
		bcs .rollover
		sta $6002
		;lda _y_scroll+1
		;adc #$00
		;sta $6003
		st2 #$00
		bra .sprite_zero
.rollover
		sec
		sbc #$f0
		sta $6002
		st2 #$00
.sprite_zero
		lda sprite_zero
		beq .end
		clc
		adc #$47
		st0 #$06
		sta $6002
		cla
		adc #$00
		sta $6003
		lda #$01
.end
		sta _active_xy		
		sta _split_mode
		lda _vdc_reg
		sta $6000
    pla
    rti
		
    
.split_mode
		lda _ppu_reg2
		ora #$70
		sta _ppu_reg2
		lda _vdc_reg
		sta $6000
		stz _split_mode
		pla
		rti

    
    
.vblank                       ;Screen inactive - set flag and call NMI if enabled
		;inc frame_counter_internal
.TMR_phone_home	
		stz $7403
		stz $6c01									;reset timer
		lda #$01
		sta $6c01
		stz apu_flag							;reset APU flag for timer
		stz _split_mode
		stz _active_xy
		st0 #$05
		st1 #$8c
		st0 #$06
		st1 #$48
		st2 #$00
		st0 #$07
		st1 #$00
		tst #$01, _ppu_reg0
		bne .tbl_offset1
		st2 #$00
		bra .cont1
.tbl_offset1
		st2 #$01
.cont1		
		st0 #$08
		st1 #$00
		st2 #$01
		lda _vdc_reg
		sta $6000
    lda _ppu_reg2
    ora #$80
    sta _ppu_reg2
    lda _internal_halt
    bne .out
		lda _NMI_routine
		bne .out
    lda _ppu_reg0
    and #$80
    beq .out

		pla
		sta _VStemp0
    lda #HIGH(_NMI_RETURN)
    pha
    lda #LOW(_NMI_RETURN)
    pha
    php												; need to setup for an RTI from NES NMI routine
    
    lda #$01
    sta _NMI_routine
    lda _VStemp0
    cli                       ; that's important ;)
    jmp [_NES_NMI]            ; continue to game NMI vector
        
.out
		lda _vdc_reg
		sta $6000
    pla    
    rti

_NMI_RETURN:
		sei
		stz _NMI_routine
		rti

;################################################################################
;
; Some LUTs here
;

	.include "spr_addr.lut"
	.include "spr_attrib.lut"
	.include "attr_addr.lut"
	.include "name_table.lut"
