; Robo Warrior 1 NES to PCE rom hack.
; Version 0.27
; 
; Assemble with PCEAS 3.21 that comes with HuC (www.zeograd.com)
;


;############################################## 
;
;
;
  
    ;internal variables 
 
  .include "VAR.asm"

  .code
  .bank $00
  .org $e000
  
_RESET: 
    sei                       ;you better know this one ;)
    lda #$f8                  
    tam #$01                  ;map ram
    tam #$00
    lda #$ff                              
    tam #$03                  ;map hardware bank
    csh                       ;set 7.16mhz mode
    cld                       ;clear decimal flag 
    ldx #$ff                  ;initialize stack
    txs

    lda $6000                 ;clear pending VDC interrupts
    st0 #$05                  ;enabled vblank/hysnc flag; disabled BG/spr
    st1 #$0c
    st2 #$00
    st0 #$06                  ;setup hsync for first active scanline
    st1 #$40
    st2 #$00
    stz $7403                 ;clear pending timer interrupts
    lda #$F8                  ;enable interrupts via mask
    sta $7402
    lda #$39
    sta $6c00                 ;set timer to 59392 cycles
    lda #$01
    sta $6c01                 ;enable TIMER

    clx
.loop                         ;initialize audio hardware
    stx $6800
    stz $6801
    stz $6802
    stz $6803
    stz $6804
    stz $6805
    stz $6806
    stz $6807
    stz $6808
    stz $6809
    inx
    cpx #$06
    bcc .loop
 
    lda #$04                  ;initialize the VCE
    sta $6400
    
    stz $0000
    tii $0000, $0001, $2000   ;clear ram
    st0 #$00
    st1 #$00
    st2 #$00
    st0 #$02
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram
    tia $0000, $6002, $2000   ;clear vram

    st0 #$00
    st1 #$00
    st2 #$08
    st0 #$02
    clx
.loop1
    st1 #$80
    st2 #$00
    st1 #$80
    st2 #$00
    dex
    bne .loop1
    
    st0 #$00
    st1 #$00
    st2 #$00
    st0 #$02
    jsr init_VDC              ;init display parameters
    lda #$01        
    tam #$02                  ;setup our back end code bank to $4000
    stz _spr_lsb
    lda #$7F
    sta _spr_msb
    st0 #$0f
    st1 #$10
    st2 #$00
    
    lda #$ff
    sta $6801
    stz $6800    
    lda #$40
    sta $6804
    stz $6804
    tin duty_cycle0,$6806,$20 ;update waveform buffer
    lda #$cc
    sta $6805
    lda #$01
    sta $6800
    lda #$40
    sta $6804
    stz $6804
    tin duty_cycle0,$6806,$20 ;update waveform buffer
    lda #$cc
    sta $6805
    lda #$02
    sta $6800
    lda #$40
    sta $6804
    stz $6804
    tin triangle_cycle,$6806,$20  ;update waveform buffer
    lda #$ff
    sta $6805
    lda #$05
    sta $6800
    stz $804
    lda #$99
    sta $6805
    
    lda #$39
    sta $6c00                 ;set timer to 59392 cycles
    lda #$01
    sta $6c01                 ;enable TIMER
  
    jmp __prep_rom            ;Initialization done.
  
init_VDC:
  .include "INIT_VDC.asm"
    
    ;// Interrupt startup handlers

_IRQ_2:
    rti
    
_TIMER:
    pha
    lda #$ff
    tam #$00
    stz $1403
    lda #$f8
    tam #$00
    pla
    rti

_VDC_INT:
    pha
    lda #$ff
    tam #$00
    lda $0000
    lda #$f8
    tam #$00
    pla
    rti
  
_NMI:
    rti
    
  .org $fff6
  
    ;// vector_table

    .dw _IRQ_2
    .dw _VDC_INT
    .dw _TIMER
    .dw _NMI
    .dw _RESET




;############################################## 

  .bank $01
  .org $4000
    
__prep_rom:

    lda #$00                  ;setup startup lower bank
    jsr MPR_REG
    lda #FIXED_BANK           ;setup startup fixed upper bank
    tam #$06
    inc a
    tam #$07
    jmp [$fffe]               ;jump to NES reset vector 
    
    .org $4040

  .include "PPU.asm"
  .include "MAPPER.asm"
  .include "IO.asm"
  .include "APU.asm"
  .include "EQUATE.asm"




;############################################## 
;
; Load the rom into the project file. The file
; allocates banks $02 to $11 (128k).
;
  .bank $02
    
rwarrior_rom:  
  .incbin "robo_warrior.bin"
  
                
  

;############################################## 
;
; This section is for the rom hardware interface modifications.
; This will overwrite sections of "megaman.bin" and allow us
; to keep track of the changes in a easy to locate format.
;

  .bank $0c   ;################################
  
  .org $818d
      jsr APU_4000_iy
  
  .org $81b4
      jsr APU_4002_iy
      
  .org $8342
      jsr APU_4001_iy
      
  .org $836e
      jsr APU_4002_iy
      
  .org $837d
      jsr APU_4003_iy
      
  .org $8387
      jsr APU_4008
      
  .org $8353
      jsr APU_4006
      
  .org $8362
      jsr APU_4007
      
  .org $8313
      jsr APU_400C_iy
      
  .org $86d6
      jsr APU_4011
      
  .org $86e5
      jsr APU_4008
      jsr APU_400B
      
  .org $86ed
      jsr APU_4000
      jsr APU_4004
      jsr APU_400C
      
  .org $8053
      jsr APU_4000
      jsr APU_4004
      jsr APU_400C

  .org $805e
      jsr APU_4008
      jsr APU_400A
      jsr APU_400B

  .org $8024
      jsr APU_4000
      jsr APU_4004
      jsr APU_400C

  .org $802f
      jsr APU_4008
      jsr APU_4003
      jsr APU_4007
      jsr APU_400B
      jsr APU_400F
      
  .org $90de
      jsr APU_4004
      jsr APU_400C
      
  .org $90e9
      jsr APU_4004
      jsr APU_400C
      
  .org $90f1
      jsr APU_4006
      
  .org $90f6
      jsr APU_400E
      
  .org $90fb
      jsr APU_4005
      jsr APU_4007
      jsr APU_400F
      
  .org $9132
      jsr APU_400C
      jsr APU_4004
      
  .org $913b
      jsr APU_400E
      
  .org $9140
      jsr APU_4006
      
  .org $9145
      jsr APU_4005
      jsr APU_4007
      jsr APU_400F
      
  .org $911c
      jsr APU_400C
      
  .org $917b
      jsr APU_4000
      
  .org $9180
      jsr APU_4004
      
  .org $918a
      jsr APU_4006
      
  .org $9190
      jsr APU_4002
      
  .org $9195
      jsr APU_4001
      jsr APU_4005
      
  .org $919d
      jsr APU_4003
      jsr APU_4007
      
  .org $9170
      jsr APU_4000
      jsr APU_4004
      
  .org $9241    
      jsr APU_400C
      
  .org $924d
      jsr APU_400E
      
  .org $9252
      jsr APU_400F
      
  .org $91cb
      jsr APU_4003
      jsr APU_4007
      jsr APU_4001
      jsr APU_4005
      
  .org $91d9
      jsr APU_400C
      
  .org $91fa
      jsr APU_4000
      jsr APU_4004
      
  .org $9202
      jsr APU_400C
      
  .org $920e
      jsr APU_400E
      
  .org $9213
      jsr APU_400F    
      
  .org $9218
      jsr APU_4002
      
  .org $921e
      jsr APU_4006
      
  .org $878e
      jsr APU_4004
      
  .org $8793
      jsr APU_4005
      
  .org $8799
      jsr APU_4006
      
  .org $879f
      jsr APU_4006
      
  .org $87a4
      jsr APU_4007
      
  .org $8786
      jsr APU_4004
      
  .org $8713
      jsr APU_4004
  
  .org $8718
      jsr APU_4005

  .org $871e
      jsr APU_4006
      
  .org $8724
      jsr APU_4007
      
  .org $8742
      jsr APU_400C
      
  .org $8747
      jsr APU_400F
  
  .org $874f
      jsr APU_400E
      
  .org $886b
      jsr APU_4004
      
  .org $8870
      jsr APU_4005

  .org $8889
      jsr APU_4006
      
  .org $888e
      jsr APU_4007
      
  .org $8863
      jsr APU_4004
      
  .org $87d7
      jsr APU_4006
      
  .org $87dd
      jsr APU_4007
      
  .org $87e2
      jsr APU_4004
      
  .org $8923
      jsr APU_4004
      
  .org $8928
      jsr APU_4005

  .org $892e
      jsr APU_4006
      
  .org $8934
      jsr APU_4007
      
  .org $8918
      jsr APU_4004
  
  .org $89c9
      jsr APU_4004
      
  .org $89ce
      jsr APU_4005
      
  .org $89df
      jsr APU_4006
      
  .org $89e4
      jsr APU_4007
      
  .org $89fa
      jsr APU_4004
      
  .org $88c5
      jsr APU_4000

  .org $88ca
      jsr APU_4001
      
  .org $88cf
      jsr APU_4002
      
  .org $88d4
      jsr APU_4003
  
  .org $88de
      jsr APU_4004
      
  .org $88e3
      jsr APU_4005
      
  .org $88e8
      jsr APU_4006
  
  .org $88ed
      jsr APU_4007
      
  .org $8cfb
      jsr APU_4000
      jsr APU_4004
  
  .org $8d03
      jsr APU_4001
      jsr APU_4005
      
  .org $8d0c
      jsr APU_4002
      
  .org $8d12
      jsr APU_4006
      
  .org $8d18
      jsr APU_4003
      
  .org $8d1e
      jsr APU_4007
      
  .org $8a24
      jsr APU_4004
      
  .org $8a29
      jsr APU_4005

  .org $8a2f
      jsr APU_4006
      
  .org $8a34
      jsr APU_4007
      
  .org $8a19
      jsr APU_4004
      
  .org $8823
      jsr APU_4004
  
  .org $8828
      jsr APU_4005

  .org $882e
      jsr APU_4006
  
  .org $8836
      jsr APU_4007
      
  .org $8a7c
      jsr APU_4000
      jsr APU_4004
      
  .org $8a84
      jsr APU_4001
      jsr APU_4005
      
  .org $8ab2
      jsr APU_4006
      
  .org $8aba
      jsr APU_4002
      
  .org $8abf
      jsr APU_4003
      jsr APU_4007
      
  .org $8edf
      jsr APU_4004
      
  .org $8ee4
      jsr APU_4005
      
  .org $8eea
      jsr APU_4006
  
  .org $8ef2
      jsr APU_4007
      
  .org $8d44
      jsr APU_400C
      
  .org $8d49
      jsr APU_400F
      
  .org $8d51
      jsr APU_400E
      
  .org $8e28
      jsr APU_4004
      
  .org $8e2d
      jsr APU_4005
      
  .org $8e32
      jsr APU_4006
      
  .org $8e37
      jsr APU_4007
      
  .org $8e3c
      jsr APU_400C
      
  .org $8e44
      jsr APU_400E
      
  .org $8e49
      jsr APU_400F
      
  .org $8f35
      jsr APU_4004
      
  .org $8f3d
      jsr APU_4004
      
  .org $8f42
      jsr APU_4005

  .org $8f48
      jsr APU_4006
      
  .org $8f50
      jsr APU_4007
      
  .org $8f74
      jsr APU_4004
      
  .org $8f79
      jsr APU_4005
      
  .org $8f7f
      jsr APU_4006
      
  .org $8f85
      jsr APU_4007
  
  .org $8f94
      jsr APU_4004
      
  .org $8fc5
      jsr APU_4004
      jsr APU_400C
      
  .org $8fd3
      jsr APU_4004
      jsr APU_400C
      
  .org $8fe0
      jsr APU_4005
      
  .org $8fe6
      jsr APU_4006
      
  .org $8feb
      jsr APU_400E
  
  .org $8ff0
      jsr APU_400F
      
  .org $8ff6
      jsr APU_4007
      
  .org $9027
      jsr APU_4000
      jsr APU_4004

  .org $9037
      jsr APU_4000
      jsr APU_4004
      
  .org $903f
      jsr APU_4001
      jsr APU_4005

  .org $904e
      jsr APU_4002
      
  .org $9054
      jsr APU_4006
      
  .org $9059
      jsr APU_4003
      jsr APU_4007
      
  .org $9093
      jsr APU_4004
      
  .org $90a5
      jsr APU_4004
      
  .org $90aa
      jsr APU_4005

  .org $90b0
      jsr APU_4006
      
  .org $90b6
      jsr APU_4007




  

      

      




  
      
      


  


  .bank $0e   ;################################
  
  .org $900e
      jsr PPU_2006
  
  .org $9013
      jsr PPU_2006    
      
  .org $9018
      jsr PPU_2007
      
  .org $9020
      jsr PPU_2007_x
      jsr PPU_2007
      
  .org $908b
      jsr PPU_2006
      jsr PPU_2006_x
  
  .org $9095
      jsr PPU_2007      
      

  .bank $0f   ;################################

  .org $aa04
      jsr PPU_2007

  .org $a9df
      jsr PPU_2006
      
  .org $a9ea
      jsr PPU_2006



  .bank $10   ;################################
  
  .org $c0d0
      nop                     ;sei
      
  .org $c0d4
      jsr PPU_REG0
      
  .org $c0db
      jsr PPU_REG1
      
  .org $c0e3
      jsr PPU_REG2

  .org $c0e8
      jsr PPU_REG2

  .org $ccb9
      jsr PPU_REG2
      
  .org $ccbe
      jsr PPU_REG2
  
  .org $cce2
      jsr PPU_REG0
      
  .org $ccb3
      jsr PPU_REG1
      
  .org $ccf4
      lda mapper_table,x      ;lda $fff0,x
      jsr MPR_REG_X           ;sta $fff0,x
      

  .org $ccea
      lda mapper_table,x      ;lda $fff0,x
      jsr MPR_REG_X           ;sta $fff0,x
    
  .org $cf8e
      jsr PPU_REG0
  
  .org $cfbc
      jsr PPU_REG1
  
  .org $cf84
      jsr PPU_2005
      jsr PPU_2005
  
  .org $cc7c
      jsr PPU_2006            ;sta $2006
      jsr PPU_2006_x          ;stx $2006
  
  .org $cc84
      jsr PPU_2005            ;sta $2005
      jsr PPU_2005            ;sta $2005
  
  .org $cc8e
      jsr PPU_REG0            ;sta $2000
      
  .org $cc78
      jsr PPU_2007            ;sta $2007
  
  .org $cc9e
      jsr PPU_REG3            ;sta $2003
 
   .org $d159
      jsr PPU_2006
      jsr PPU_2006_x
      
  .org $d160
      jsr PPU_2007_R
      jsr PPU_2007_R
      
  .org $d171
      jsr PPU_2006
      jsr PPU_2006_x
      
  .org $d178
      jsr PPU_2007
      
  .org $d17f
      jsr PPU_2005
      jsr PPU_2005
      
  .org $d189
      jsr PPU_REG0

  .org $d19a
      jsr PPU_2006
      jsr PPU_2006_x
      
  .org $d1a2
      jsr PPU_2005
      jsr PPU_2005
      
  .org $d1af
      jsr PPU_REG0
      
  .org $d1b7
      jsr PPU_2007

  .org $d1be
      jsr PPU_2007

  .org $d1c5
      jsr PPU_2007

  .org $d1cc
      jsr PPU_2007

  .org $d1d3
      jsr PPU_2007

  .org $d1da
      jsr PPU_2007

  .org $d1e1
      jsr PPU_2007

  .org $d1e8
      jsr PPU_2007

  .org $d1ef
      jsr PPU_2007

  .org $d1f6
      jsr PPU_2007

  .org $d1fd
      jsr PPU_2007

  .org $d204
      jsr PPU_2007

  .org $d20b
      jsr PPU_2007

  .org $d212
      jsr PPU_2007

  .org $d219
      jsr PPU_2007

  .org $d220
      jsr PPU_2007

  .org $d227
      jsr PPU_2007

  .org $d22e
      jsr PPU_2007

  .org $d235
      jsr PPU_2007

  .org $d23c
      jsr PPU_2007

  .org $d243
      jsr PPU_2007

  .org $d24a
      jsr PPU_2007

  .org $d251
      jsr PPU_2007

  .org $d258
      jsr PPU_2007

  .org $d25f
      jsr PPU_2007

  .org $d266
      jsr PPU_2007

  .org $d26d
      jsr PPU_2007

  .org $d274
      jsr PPU_2007

  .org $d27b
      jsr PPU_2007

  .org $d282
      jsr PPU_2007

  .org $d289
      jsr PPU_2007

  .org $d290
      jsr PPU_2007

  .org $d296
      jsr PPU_2006

  .org $d29b
      jsr PPU_2006
      jsr PPU_2006
      jsr PPU_2006

  .org $d2a6
      jsr PPU_2005
      jsr PPU_2005

  .org $d2b0
      jsr PPU_REG0

  .org $cd14
      jsr PPU_2006
      
  .org $cd19
      jsr PPU_2006
      jsr PPU_2006
      jsr PPU_2006
      
  .org $cca3
      jsr SPR_DMA
      
  .org $d9ac
      jsr PPU_REG0
  
  .org $d9b8
      jsr PPU_2007
      
  .org $d9be
      jsr PPU_2007
      
  .org $d9cf
      jsr PPU_2007
      
  .org $d9d5
      jsr PPU_2007
      
  .org $d9e0
      jsr PPU_2007_R
      jsr PPU_2007_R
  
  .org $d9f0
      jsr PPU_2007

  .org $da05
      jsr PPU_2007

  .org $da29
      jsr PPU_2007
      
  .org $da33
      jsr PPU_2007
      
  .org $da38
      jsr PPU_2007
      
  .org $da41
      jsr PPU_2007

  .org $d33f
      jsr PPU_REG0
  
  .org $d343
      jsr PPU_2007
        
  .org $d363
      jsr PPU_2007
  
  .org $d36d
      jsr PPU_2005
      jsr PPU_2005
      
  .org $d377
      jsr PPU_REG0
      
  .org $d396
      jsr PPU_2007
  
  .org $d39d
      jsr PPU_2007
      
  .org $d3a8
      jsr PPU_2005
      jsr PPU_2005
  
  .org $d3b2
      jsr PPU_REG0
      
  .org $d3f0                  ;temporary
      nop
      nop

  .org $d3f5                  ;temporary
      nop
      nop
  
  .org $d401
      jsr PPU_2005
      
  .org $d406
      jsr PPU_2005
      
  .org $d417
      jsr PPU_REG0
  
  .org $d2bb
      jsr PPU_REG0
      
  .org $d2c7
      jsr PPU_2006
      jsr PPU_2006_x
      
  .org $d2cf
      jsr PPU_2005
      jsr PPU_2005

  .org $d2d9
      jsr PPU_REG0      
      
  .org $d2e0
      jsr PPU_2007
      
  .org $d2e7
      jsr PPU_2007
      
  .org $d2f3
      jsr PPU_2006
      jsr PPU_2006_x
      
  .org $d2fd
      jsr PPU_2007
      
  .org $d304
      jsr PPU_2007
      
  .org $d314
      jsr PPU_2006
      jsr PPU_2006_x
      jsr PPU_2007_R
      jsr PPU_2007_R

  .org $d32d
      jsr PPU_2006
      jsr PPU_2006_x

  .org $d335
      jsr PPU_2005
      jsr PPU_2005
      
  .org $d053
      jsr PPU_2007
      
  .org $d05d
      jsr PPU_2005
      jsr PPU_2005

  .org $d067
      jsr PPU_REG0      
  
  .org $d071
      jsr PPU_REG0

  .org $d07e
      jsr PPU_2006
      jsr PPU_2006_x
      
  .org $d088
      jsr PPU_2007    
      
  .org $d088
      jsr PPU_2007    
      
  .org $d08f
      jsr PPU_2007    
      
  .org $d096
      jsr PPU_2007    
      
  .org $d09d
      jsr PPU_2007    
      
  .org $d0a4
      jsr PPU_2007    
      
  .org $d0ab
      jsr PPU_2007    
      
  .org $d0b2
      jsr PPU_2007    
      
  .org $d0b9
      jsr PPU_2007    
      
  .org $d0c0
      jsr PPU_2007    
      
  .org $d0c7
      jsr PPU_2007    
      
  .org $d0ce
      jsr PPU_2007    
      
  .org $d0d5
      jsr PPU_2007    
      
  .org $d0dc
      jsr PPU_2007    
      
  .org $d0e3
      jsr PPU_2007    
      
  .org $d0ea
      jsr PPU_2007    
      
  .org $d0f1
      jsr PPU_2007    
      
  .org $d0f8
      jsr PPU_2007    

  .org $d0ff
      jsr PPU_2007    
      
  .org $d106
      jsr PPU_2007    
      
  .org $d10d
      jsr PPU_2007    
      
  .org $d0ab
      jsr PPU_2007    
      
  .org $d114
      jsr PPU_2007    
      
  .org $d11b
      jsr PPU_2007    
      
  .org $d122
      jsr PPU_2007    
      
  .org $d129
      jsr PPU_2007    
      
  .org $d130
      jsr PPU_2007    
      
  .org $d137
      jsr PPU_2007    
      
  .org $d13e
      jsr PPU_2005
      jsr PPU_2005
  
  .org $d148
      jsr PPU_REG0
      
  .org $c578
      jsr PPU_2005
      
  .org $c57d
      jsr PPU_2005  
      
  .org $c587
      jsr PPU_2005
      
  .org $c58c
      jsr PPU_2005
      
  .org $da4b          ;changed from $da48
      jsr PPU_2007
      jsr PPU_2007    
  
  .org $cfcb
      jsr IO_PORT
      lda __gamepad
      bra $cfec
  
  

   
  .bank $11   ;################################

  .org $FFF8                  ;setup internal VDC int
    .dw _INT_VDC
  .org $FFF6                  ;remap BRK to PCE vector
    .dw $fff9
  .org $FFFA                  ;setup internal TMR int
    .dw _INT_TMR
  .org $FFFC                  ;remap NMI to PCE NMI which not used. 
    _NES_NMI: .dw $cf75       ;We'll use this as an indirect jump
  .org $FFFE
    .dw $c0d0                 ;remap reset to PCE vector

  .org $fbf0
      jsr PPU_2007
      
  .org $f396
      jsr PPU_2007
      
  .org $f39c
      jsr PPU_2007
      
  .org $f3a2
      jsr PPU_2007
      
  .org $f3a8
      jsr PPU_2007
      
  .org $f3bb
      jsr PPU_2007
      
  .org $f3c1
      jsr PPU_2007
      
  .org $f3c7
      jsr PPU_2007
      
  .org $f3cd
      jsr PPU_2007
      
  .org $e91f
      jsr PPU_2006
      
  .org $e92c
      jsr PPU_2007_x
      jsr PPU_2007
      
  .org $e934
      jsr PPU_2006
      
  .org $e939
      jsr PPU_2006
      
  .org $e942
      jsr PPU_2007_x
      jsr PPU_2007
      


;############################################## 
;
; Pad rom to 512k
;
  .bank $3f                 ;pad to last bank
  .bank $3e                 ;re-align back to LUT bank

    ;LUTs go here.














